import { create } from 'd3-selection';
import { scaleLinear } from 'd3-scale';
import { min, max } from 'd3-array';
import { axisLeft, axisBottom } from 'd3-axis';

export const Histogram = ({
  data,
  xMinAccessor,
  xMaxAccessor,
  xAxisLabel,
  xAxisLabelOffset,
  yAccessor,
  yAxisLabel,
  yAxisLabelOffset,
  margin,
  width,
  height,
}) => {
  const svg = create('svg')
    .attr('width', width)
    .attr('height', height);

  const xScale = scaleLinear()
    .domain([
      min(data, xMinAccessor),
      max(data, xMaxAccessor),
    ])
    .range([margin.left, width - margin.right]);

  const yScale = scaleLinear()
    .domain([0, max(data, yAccessor)])
    .range([height - margin.bottom, margin.top]);

  svg
    .selectAll('rect')
    .data(data)
    .join('rect')
    .attr('x', (d) => xScale(xMinAccessor(d)))
    .attr(
      'width',
      (d) =>
        xScale(xMaxAccessor(d)) - xScale(xMinAccessor(d))
    )
    .attr('y', (d) => yScale(yAccessor(d)))
    .attr('height', (d) => yScale(0) - yScale(yAccessor(d)))
    .attr('fill', (d) => (d.max <= 10 ? '#CCC' : 'black'));

  svg
    .append('g')
    .attr('transform', `translate(${margin.left}, 0)`)
    .call(axisLeft(yScale));

  svg
    .append('g')
    .attr(
      'transform',
      `translate(0,${height - margin.bottom})`
    )
    .call(axisBottom(xScale));

  svg
    .append('text')
    .attr('text-anchor', 'middle')
    .attr('alignment-baseline', 'middle')
    .attr(
      'transform',
      `translate(${margin.left - yAxisLabelOffset},${
        margin.top +
        (height - margin.top - margin.bottom) / 2
      }) rotate(-90)`
    )
    .text(yAxisLabel);

  svg
    .append('text')
    .attr('text-anchor', 'middle')
    .attr('alignment-baseline', 'middle')
    .attr(
      'transform',
      `translate(${
        margin.left +
        (width - margin.right - margin.left) / 2
      },${height - margin.bottom + xAxisLabelOffset})`
    )
    .text(xAxisLabel);

  return svg.node();
};
